/*
 * Copyright (C) 2022-2026 Valve Corporation
 * Copyright (C) 2022-2026 LunarG, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Christophe Riccio <christophe@lunarg.com>
 */

#pragma once
#include <json/json.h>
#include <memory>
#include <string>

#include "profiles_settings.h"

struct JsonValidator {
    JsonValidator(){}
    ~JsonValidator();

    bool Init();

    bool Check(const Json::Value &json_document);

    std::string message;
};

bool WarnDuplicated(ProfileLayerSettings *layer_settings, const Json::Value &parent, const std::vector<std::string> &members);
